<?php

class ControllerExtensionModuleDntradeApi extends Controller {

    public $baseURL = "";

    public function __construct($registry) {
	parent::__construct($registry);

	// Включаємо відображення помилок
	ini_set('display_errors', 1);
// Встановлюємо рівень повідомлень про помилки
	error_reporting(E_ALL);

	$this->response->addHeader('Content-Type: application/json');
	$this->load->language('extension/module/dntrade_api');
	$this->load->model('extension/module/dntrade_api');
	$this->baseURL = rtrim(HTTP_SERVER, '/') . '/'; // HTTP_SERVER має містити базовий URL сайту, включаючи схему (http або https)
    }

    public function index() {
	$error = $this->auth();
	if ($error) {
	    return $this->response->setOutput(json_encode(['status' => 0, 'error' => $error]));
	}
	return $this->response->setOutput(json_encode(['status' => 1]));
    }

    public function getstatuses() {
	$error = $this->auth();
	if ($error) {
	    return $this->response->setOutput(json_encode(['status' => 0, 'error' => $error]));
	}
	// Завантажуємо модель order/order_status
	$this->load->model('localisation/order_status');

// Отримуємо список всіх статусів замовлень
	$orderStatuses = $this->model_localisation_order_status->getOrderStatuses();

// Масив для зберігання статусів у форматі, який ви зазначили
	$statusList = array();

// Перебираємо всі статуси та формуємо список
	foreach ($orderStatuses as $status) {
	    $statusList[] = array(
		'id' => $status['order_status_id'],
		'title' => $status['name']
	    );
	}
	return $this->response->setOutput(json_encode(['status' => 1, 'statuses' => $statusList]));
    }

    public function products() {
	$error = $this->auth();
	if ($error) {
	    return $this->response->setOutput(json_encode(['status' => 0, 'error' => $error]));
	}

	// Завантажити модель товарів
	$this->load->model('catalog/product');
	
	$limit = isset($this->request->post['limit']) ? $this->request->post['limit'] : 500;
	$offset = isset($this->request->post['offset']) ? $this->request->post['offset'] : 0;

	// Отримати список товарів у форматі {id, title, sku, unit_title, image, category}
	$productList = $this->getProductsList($limit, $offset);

	return $this->response->setOutput(json_encode(['status' => 1, 'data' => $productList]));
    }

    public function product() {
	$error = $this->auth();
	if ($error) {
	    return $this->response->setOutput(json_encode(['status' => 0, 'error' => $error]));
	}

	$id = isset($this->request->post['id']) ? $this->request->post['id'] : 0;
	if (empty($id)) {
	    return $this->response->setOutput(json_encode(['status' => 0, 'message' => "Параметр id обов'язковий"]));
	}
	$this->load->model('catalog/category');

	// Завантажити модель товарів
	$this->load->model('catalog/product');

	$product = $this->model_catalog_product->getProduct($id);
	return $this->response->setOutput(json_encode(['status' => 1, "id" => $id, 'product' => $this->getProductItem($product)]));
    }
    
    public function setstatus() {
	$error = $this->auth();
	if ($error) {
	    return $this->response->setOutput(json_encode(['status' => 0, 'error' => $error]));
	}

	$order_id = isset($this->request->post['order_id']) ? $this->request->post['order_id'] : 0;
	if (empty($order_id)) {
	    return $this->response->setOutput(json_encode(['status' => 0, 'message' => "Параметр order_id обов'язковий"]));
	}
	
	$status = isset($this->request->post['status']) ? $this->request->post['status'] : 0;
	if (empty($status)) {
	    return $this->response->setOutput(json_encode(['status' => 0, 'message' => "Параметр status обов'язковий"]));
	}
	
	$this->load->model('extension/module/dntrade_api');	
	$order_status = $this->model_extension_module_dntrade_api->editOrderStatus($order_id, $status);	
	return $this->response->setOutput(json_encode(['status' => (int)$order_status]));
    }

    // Функція для отримання списку товарів у форматі {id, title, sku, unit_title, image, category}
    private function getProductsList($limit, $offset) {	
	$productList = array();
	$this->load->model('catalog/category');
	$products = $this->model_catalog_product->getProducts(['start' => $offset, "limit" => $limit]);
	foreach ($products as $product) {
	    $productList[] = $this->getProductItem($product);
	}

	return $productList;
    }

    private function getProductItem($product) {
	if (empty($product))
	    return array();
	$categories = $this->model_catalog_product->getCategories($product['product_id']);
	$category = [];
	if (!empty($categories[0])) {
	    $category = $this->model_catalog_category->getCategory($categories[0]["category_id"]);
	}
	return array(
	    'id' => $product['product_id'],
	    'title' => $product['name'],
	    'code' => $product['model'],
	    'sku' => $product['sku'],
	    'quantity' => $product['quantity'],
	    'status' => $product['status'],
	    'link' => $this->url->link('product/product') . '&product_id=' . $product['product_id'],
	    'unit_title' => "шт",
	    'image' => $product['image'] ? $this->baseURL . 'image/' . $product['image'] : null, // Передбачається, що це поле містить шлях до зображення товару
	    'parent' => !empty($category) ? [
	"id" => $category["category_id"],
	'title' => $category['name'],
	'parent_id' => ($category['parent_id'] != 0) ? $category['parent_id'] : null
	    ] : NULL, // Передбачається, що це поле містить категорію або категорії товару
	    'price' => isset($product['special']) && $product['special'] > 0 ? $product['special'] : $product['price'],
	    'description' => $product['description']
	);
    }

    public function categories() {

	$error = $this->auth();
	if ($error) {
	    return $this->response->setOutput(json_encode(['status' => 0, 'error' => $error]));
	}

// Завантажити модель категорій товарів
	$this->load->model('catalog/category');

// Отримати список категорій у форматі з однорівневою структурою
	$categoryFlatList = $this->getCategoriesFlat($this->model_catalog_category);

// Вивести список у форматі JSON
	return $this->response->setOutput(json_encode(['status' => 1, 'data' => $categoryFlatList]));
    }

    public function orders() {
	$error = $this->auth();
	if ($error) {
	    return $this->response->setOutput(json_encode(['status' => 0, 'error' => $error]));
	}
	$from_date = isset($this->request->post['from_date']) ? date("Y-m-d H:i:s", strtotime($this->request->post['from_date'])) : "";
	$limit = isset($this->request->post['limit']) ? (int) $this->request->post['limit'] : 100;
	$offset = isset($this->request->post['limit']) ? (int) $this->request->post['offset'] : "";
	$this->load->model('extension/module/dntrade_api');
	$this->load->model('catalog/category');
	$this->load->model('catalog/product');
	$orders = $this->model_extension_module_dntrade_api->getOrders($from_date, $limit, $offset);

	$order_data = [];

	foreach ($orders as $order) {
	    $order_id = $order['order_id'];
	    // Отримання даних про замовлення
	    $order_info = [
		'order_id' => $order['order_id'],
		'customer_id' => $order['customer_id'],
		'customer_name' => $order['firstname'] . ' ' . $order['lastname'],
		'email' => $order['email'],
		'phone' => $order['telephone'],
		'payment_method' => $order['payment_method'],
		'address' => $order['shipping_city'] . ", " . $order['shipping_address_1'] . ($order['shipping_address_2'] ? ", " . $order['shipping_address_2'] : ""),
		'shipping_method' => $order['shipping_method'],
		'comment' => $order['comment'],
		'total' => $order['total'],
		'order_status' => $order['order_status_id'],
		'date_added' => $order['date_added'],
		'date_modified' => $order['date_modified']
	    ];

	    // Якщо потрібно, ви можете отримати інші дані про клієнта з моделі customer
	    // Отримання даних про товари у замовленні
	    $products = $this->model_extension_module_dntrade_api->getOrderProducts($order_id);

	    $order_products = [];
	    foreach ($products as $product) {
		$order_products[] = [
		    'id' => $product['product_id'],
		    'product' => $this->getProductItem($this->model_catalog_product->getProduct($product['product_id'])),
		    'quantity' => $product['quantity'],
		    'price' => $product['price'],
		    'total_price' => $product['total'],
		    'tax' => $product['tax'],
		    'reward' => $product['reward']
		];
	    }

	    $order_info['products'] = $order_products;

	    $order_data[] = $order_info;
	}
	return $this->response->setOutput(json_encode(['status' => 1, 'orders' => $order_data]));
    }

    public function upload() {
	$error = $this->auth();
	if ($error) {
	    return $this->response->setOutput(json_encode(['status' => 0, 'error' => $error]));
	}
	if (isset($_POST['products']) && !empty($_POST['products'])) { 
	    $new_ids = [];
	    $products = json_decode($_POST['products']);
	    foreach ($products as $item) {
		$this->load->model('extension/module/dntrade_api');
		$data = array();
		if (isset($item->title)) {
		    $data['name'] = $item->title;
		}
		if (isset($item->sku)) {
		    $data['sku'] = $item->sku;
		}
		if (isset($item->ean)) {
		    $data['ean'] = $item->ean;
		}
		if (isset($item->title)) {
		    $data['name'] = $item->title;
		}
		if (isset($item->price_old)) {
		    $data['price'] = $item->price_old;
		}
		if (isset($item->price)) {
		    $data['price_special'] = $item->price;
		}
		if (isset($item->quantity)) {
		    $data['quantity'] = $item->quantity;
		}
		if (isset($item->images)) {
		    $data['images'] = $item->images;
		}
		if (isset($item->images)) {
		    $data['images'] = $item->images;
		}
		if (isset($item->product_store)) {
		    $data['product_store'] = $item->product_store;
		}
		if (isset($item->parent_id)) {
		    $data['parent_id'] = $item->parent_id;
		}
		if (isset($item->description)) {
		    $data['description'] = $item->description;
		}
		if (!$item->is_new) {
		    $this->model_extension_module_dntrade_api->editProduct($item->id, $data, $this->getDefLanguage());
		} else {
		    $id = $this->model_extension_module_dntrade_api->addProduct($data, $this->getDefLanguage());
		    $new_ids[$item->system_id] = $id;
		}
	    }
	}
	return $this->response->setOutput(json_encode(['status' => 1, 'new' => $new_ids]));
    }

    private function getDefLanguage() {
	// Підключення моделі мови
	$this->load->model('localisation/language');

// Отримання мов за замовчуванням
	$languages = $this->model_localisation_language->getLanguages();

// Перевірка наявності мов
	if ($languages) {
	    foreach ($languages as $language) {
		if ($language['code'] == $this->config->get('config_language')) {
		    $language_id = $language['language_id'];
		    break;
		}
	    }
	}
	return $language_id;
    }

    // Функція для отримання категорій у форматі з однорівневою структурою
    private function getCategoriesFlat($model, $parent_id = 0) {
	$categoryList = array();
	$categories = $model->getCategories($parent_id);
	foreach ($categories as $category) {
	    $categoryImage = $category['image'] ? $this->baseURL . 'image/' . $category['image'] : NULL;
	    $categoryList[] = array(
		'id' => $category['category_id'],
		'title' => $category['name'],
		'image' => $categoryImage, // Додаємо зображення категорії
		'parent_id' => ($category['parent_id'] != 0) ? $category['parent_id'] : null
	    );

	    // Рекурсивно додаємо підкатегорії
	    $children = $this->getCategoriesFlat($model, $category['category_id']);
	    if (!empty($children)) {
		$categoryList = array_merge($categoryList, $children);
	    }
	}
	return $categoryList;
    }

    private function getTokenFromHeaders() {
	$headers = apache_request_headers(); // Отримуємо всі заголовки запиту
	if (isset($headers['Apikey'])) {
	    return $headers['Apikey']; // Витягуємо токен із заголовка Authorization
	}
	if (isset($headers['ApiKey'])) {
	    return $headers['ApiKey']; // Витягуємо токен із заголовка Authorization
	}
	if (isset($headers['apikey'])) {
	    return $headers['apikey']; // Витягуємо токен із заголовка Authorization
	}
	return null;
    }

    private function auth() {
	$enabled = $this->model_extension_module_dntrade_api->isEnabled();
	if (!$enabled) {
	    return 'Модуль DNTrade API виключено';
	}

	// Перевірка токену в заголовках запиту
	$token = $this->getTokenFromHeaders();

	// Отримання токену з бази даних
	$storedToken = $this->model_extension_module_dntrade_api->getApiToken();

	if (empty($storedToken)) {
	    return 'Токен не задано';
	}

	if (!$token || $token !== $storedToken) {
	    return 'Не авторизовано';
	}
	return false;
    }
}
